@@ -52,6 +52,7 @@ static gboolean do_poke       = FALSE;
 static gboolean do_inhibit    = FALSE;
 
 static gboolean do_query      = FALSE;
+static gboolean do_time       = FALSE;
 
 static char    *inhibit_reason      = NULL;
 static char    *inhibit_application = NULL;
@@ -61,6 +62,8 @@ static GOptionEntry entries [] = {
           N_("Causes the screensaver to exit gracefully"), NULL },
         { "query", 'q', 0, G_OPTION_ARG_NONE, &do_query,
           N_("Query the state of the screensaver"), NULL },
+        { "time", 't', 0, G_OPTION_ARG_NONE, &do_time,
+          N_("Query the length of time the screensaver has been active"), NULL },
         { "lock", 'l', 0, G_OPTION_ARG_NONE, &do_lock,
           N_("Tells the running screensaver process to lock the screen immediately"), NULL },
         { "cycle", 'c', 0, G_OPTION_ARG_NONE, &do_cycle,
@@ -84,22 +87,6 @@ static GOptionEntry entries [] = {
 
 static GMainLoop *loop = NULL;
 
-static gboolean
-screensaver_is_running (DBusConnection *connection)
-{
-        DBusError               error;
-        gboolean                exists;
-
-        g_return_val_if_fail (connection != NULL, FALSE);
-
-        dbus_error_init (&error);
-        exists = dbus_bus_name_has_owner (connection, GS_SERVICE, &error);
-        if (dbus_error_is_set (&error))
-                dbus_error_free (&error);
-
-        return exists;
-}
-
 static DBusMessage *
 screensaver_send_message_inhibit (DBusConnection *connection,
                                   const char     *application,
@@ -108,7 +95,7 @@ screensaver_send_message_inhibit (DBusConnection *connection,
         DBusMessage    *message;
         DBusMessage    *reply;
         DBusError       error;
-	DBusMessageIter iter;
+        DBusMessageIter iter;
 
         g_return_val_if_fail (connection != NULL, NULL);
 
@@ -120,9 +107,9 @@ screensaver_send_message_inhibit (DBusConnection *connection,
                 return NULL;
         }
 
-	dbus_message_iter_init_append (message, &iter);
-	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &application);
-	dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &reason);
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &application);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &reason);
 
         reply = dbus_connection_send_with_reply_and_block (connection,
                                                            message,
@@ -148,7 +135,7 @@ screensaver_send_message_bool (DBusConnection *connection,
         DBusMessage    *message;
         DBusMessage    *reply;
         DBusError       error;
-	DBusMessageIter iter;
+        DBusMessageIter iter;
 
         g_return_val_if_fail (connection != NULL, NULL);
         g_return_val_if_fail (name != NULL, NULL);
@@ -161,8 +148,8 @@ screensaver_send_message_bool (DBusConnection *connection,
                 return NULL;
         }
 
-	dbus_message_iter_init_append (message, &iter);
-	dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &value);
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_BOOLEAN, &value);
 
         reply = dbus_connection_send_with_reply_and_block (connection,
                                                            message,
@@ -221,6 +208,83 @@ screensaver_send_message_void (DBusConnection *connection,
         return reply;
 }
 
+static char **
+get_string_from_iter (DBusMessageIter *iter,
+                      int             *num_elements)
+{
+        int    count;
+        char **buffer;
+
+        if (num_elements != NULL) {
+                *num_elements = 0;
+        }
+
+        count = 0;
+        buffer = (char **)malloc (sizeof (char *) * 8);
+
+        if (buffer == NULL) {
+                goto oom;
+        }
+
+        buffer[0] = NULL;
+        while (dbus_message_iter_get_arg_type (iter) == DBUS_TYPE_STRING) {
+                const char *value;
+                char       *str;
+
+                if ((count % 8) == 0 && count != 0) {
+                        buffer = realloc (buffer, sizeof (char *) * (count + 8));
+                        if (buffer == NULL) {
+                                goto oom;
+                        }
+                }
+
+                dbus_message_iter_get_basic (iter, &value);
+                str = strdup (value);
+                if (str == NULL) {
+                        goto oom;
+                }
+
+                buffer[count] = str;
+
+                dbus_message_iter_next (iter);
+                count++;
+        }
+
+        if ((count % 8) == 0) {
+                buffer = realloc (buffer, sizeof (char *) * (count + 1));
+                if (buffer == NULL) {
+                        goto oom;
+                }
+        }
+
+        buffer[count] = NULL;
+        if (num_elements != NULL) {
+                *num_elements = count;
+        }
+        return buffer;
+
+oom:
+        g_debug ("%s %d : error allocating memory\n", __FILE__, __LINE__);
+        return NULL;
+
+}
+
+static gboolean
+screensaver_is_running (DBusConnection *connection)
+{
+        DBusError               error;
+        gboolean                exists;
+
+        g_return_val_if_fail (connection != NULL, FALSE);
+
+        dbus_error_init (&error);
+        exists = dbus_bus_name_has_owner (connection, GS_SERVICE, &error);
+        if (dbus_error_is_set (&error))
+                dbus_error_free (&error);
+
+        return exists;
+}
+
 static gboolean
 do_command (DBusConnection *connection)
 {
@@ -233,8 +297,14 @@ do_command (DBusConnection *connection)
 
         if (do_query) {
                 DBusMessageIter iter;
+                DBusMessageIter array;
                 dbus_bool_t     v;
 
+                if (!screensaver_is_running (connection)) {
+                        g_message ("Screensaver is not running!");
+                        goto done;
+                }
+
                 reply = screensaver_send_message_void (connection, "GetActive", TRUE);
                 if (! reply) {
                         g_message ("Did not receive a reply from the screensaver.");
@@ -243,9 +313,63 @@ do_command (DBusConnection *connection)
 
                 dbus_message_iter_init (reply, &iter);
                 dbus_message_iter_get_basic (&iter, &v);
-                g_print (_("The screensaver is %s\n"), v ? _("active") : _("inactive"));
+                g_print (v ? _("The screensaver is active\n") : _("The screensaver is inactive\n"));
 
                 dbus_message_unref (reply);
+
+                reply = screensaver_send_message_void (connection, "GetInhibitors", TRUE);
+                if (! reply) {
+                        g_message ("Did not receive a reply from screensaver.");
+                        goto done;
+                }
+
+                dbus_message_iter_init (reply, &iter);
+                dbus_message_iter_recurse (&iter, &array);
+
+                if (dbus_message_iter_get_arg_type (&array) == DBUS_TYPE_INVALID) {
+                        g_print (_("The screensaver is not inhibited\n"));
+                } else {
+                        char **inhibitors;
+                        int    i;
+                        int    num;
+
+                        g_print (_("The screensaver is being inhibited by:\n"));
+                        inhibitors = get_string_from_iter (&array, &num);
+                        for (i = 0; i < num; i++) {
+                                g_print ("\t%s\n", inhibitors[i]);
+                        }
+                        g_strfreev (inhibitors);
+                }
+
+                dbus_message_unref (reply);
+        }
+
+        if (do_time) {
+                DBusMessageIter iter;
+                dbus_bool_t     v;
+                dbus_int32_t    t;
+
+                reply = screensaver_send_message_void (connection, "GetActive", TRUE);
+                if (! reply) {
+                        g_message ("Did not receive a reply from the screensaver.");
+                        goto done;
+                }
+
+                dbus_message_iter_init (reply, &iter);
+                dbus_message_iter_get_basic (&iter, &v);
+                dbus_message_unref (reply);
+
+                if (v) {
+
+                        reply = screensaver_send_message_void (connection, "GetActiveTime", TRUE);
+                        dbus_message_iter_init (reply, &iter);
+                        dbus_message_iter_get_basic (&iter, &t);
+                        g_print (_("The screensaver has been active for %d seconds.\n"), t);
+
+                        dbus_message_unref (reply);
+                } else {
+                        g_print (_("The screensaver is not currently active.\n"));
+                }
         }
 
         if (do_lock) {
@@ -349,15 +473,10 @@ main (int    argc,
 
         dbus_connection_setup_with_g_main (connection, NULL);
 
-        if (! screensaver_is_running (connection)) {
-                g_message ("Screensaver is not running!");
-                exit (1);
-        }
-
         g_idle_add ((GSourceFunc)do_command, connection);
 
         loop = g_main_loop_new (NULL, FALSE);
         g_main_loop_run (loop);
 
-	return 0;
+        return 0;
 }
