@@ -33,7 +33,6 @@
 #include <glib/gi18n.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 
 #include <gio/gio.h>
 
@@ -45,20 +44,19 @@
 
 #define GTK_BUILDER_FILE "gnome-screensaver-preferences.ui"
 
-#define GNOME_LOCKDOWN_DIR  "/desktop/gnome/lockdown"
-#define KEY_LOCK_DISABLE    GNOME_LOCKDOWN_DIR "/disable_lock_screen"
+#define LOCKDOWN_SETTINGS_SCHEMA  "org.gnome.desktop.lockdown"
+#define KEY_LOCK_DISABLE    "disable-lock-screen"
 
-#define KEY_DIR             "/apps/gnome-screensaver"
-#define GNOME_SESSION_DIR   "/desktop/gnome/session"
-#define KEY_LOCK            KEY_DIR "/lock_enabled"
-#define KEY_IDLE_ACTIVATION_ENABLED         KEY_DIR "/idle_activation_enabled"
-#define KEY_MODE            KEY_DIR "/mode"
-#define KEY_ACTIVATE_DELAY  GNOME_SESSION_DIR "/idle_delay"
-#define KEY_LOCK_DELAY      KEY_DIR "/lock_delay"
-#define KEY_CYCLE_DELAY     KEY_DIR "/cycle_delay"
-#define KEY_THEMES          KEY_DIR "/themes"
+#define GS_SETTINGS_SCHEMA  "org.gnome.screensaver"
+#define KEY_LOCK            "lock-enabled"
+#define KEY_IDLE_ACTIVATION_ENABLED         "idle-activation-enabled"
+#define KEY_MODE            "mode"
+#define KEY_ACTIVATE_DELAY  "idle-delay"
+#define KEY_LOCK_DELAY      "lock-delay"
+#define KEY_CYCLE_DELAY     "cycle-delay"
+#define KEY_THEMES          "themes"
 
-#define GPM_COMMAND "gnome-power-preferences"
+#define GPM_COMMAND "gnome-control-center power"
 
 enum {
         NAME_COLUMN = 0,
@@ -66,13 +64,6 @@ enum {
         N_COLUMNS
 };
 
-static GConfEnumStringPair mode_enum_map [] = {
-       { GS_MODE_BLANK_ONLY,       "blank-only" },
-       { GS_MODE_RANDOM,           "random"     },
-       { GS_MODE_SINGLE,           "single"     },
-       { 0, NULL }
-};
-
 /* Drag and drop info */
 enum {
         TARGET_URI_LIST,
@@ -92,18 +83,17 @@ static GSJob          *job = NULL;
 static gint32
 config_get_activate_delay (gboolean *is_writable)
 {
-        GConfClient *client;
+        GSettings *client;
         gint32       delay;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
         if (is_writable) {
-                *is_writable = gconf_client_key_is_writable (client,
-                                                             KEY_ACTIVATE_DELAY,
-                                                             NULL);
+                *is_writable = g_settings_is_writable (client,
+                                                       KEY_ACTIVATE_DELAY);
         }
 
-        delay = gconf_client_get_int (client, KEY_ACTIVATE_DELAY, NULL);
+        delay = g_settings_get_int (client, KEY_ACTIVATE_DELAY);
 
         if (delay < 1) {
                 delay = 1;
@@ -117,11 +107,11 @@ config_get_activate_delay (gboolean *is_writable)
 static void
 config_set_activate_delay (gint32 timeout)
 {
-        GConfClient *client;
+        GSettings *client;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
-        gconf_client_set_int (client, KEY_ACTIVATE_DELAY, timeout, NULL);
+        g_settings_set_int (client, KEY_ACTIVATE_DELAY, timeout);
 
         g_object_unref (client);
 }
@@ -129,25 +119,17 @@ config_set_activate_delay (gint32 timeout)
 static int
 config_get_mode (gboolean *is_writable)
 {
-        GConfClient *client;
+        GSettings *client;
         int          mode;
-        char        *string;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
         if (is_writable) {
-                *is_writable = gconf_client_key_is_writable (client,
-                                                             KEY_MODE,
-                                                             NULL);
+                *is_writable = g_settings_is_writable (client,
+                                                       KEY_MODE);
         }
 
-        string = gconf_client_get_string (client, KEY_MODE, NULL);
-        if (string) {
-                gconf_string_to_enum (mode_enum_map, string, &mode);
-                g_free (string);
-        } else {
-                mode = GS_MODE_BLANK_ONLY;
-        }
+        mode = g_settings_get_enum (client, KEY_MODE);
 
         g_object_unref (client);
 
@@ -157,13 +139,11 @@ config_get_mode (gboolean *is_writable)
 static void
 config_set_mode (int mode)
 {
-        GConfClient *client;
-        const char  *mode_string;
+        GSettings *client;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
-        mode_string = gconf_enum_to_string (mode_enum_map, mode);
-        gconf_client_set_string (client, KEY_MODE, mode_string, NULL);
+        g_settings_set_enum (client, KEY_MODE, mode);
 
         g_object_unref (client);
 }
@@ -171,22 +151,20 @@ config_set_mode (int mode)
 static char *
 config_get_theme (gboolean *is_writable)
 {
-        GConfClient *client;
+        GSettings *client;
         char        *name;
         int          mode;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
         if (is_writable) {
                 gboolean can_write_theme;
                 gboolean can_write_mode;
 
-                can_write_theme = gconf_client_key_is_writable (client,
-                                                                KEY_THEMES,
-                                                                NULL);
-                can_write_mode = gconf_client_key_is_writable (client,
-                                                               KEY_MODE,
-                                                               NULL);
+                can_write_theme = g_settings_is_writable (client,
+                                                          KEY_THEMES);
+                can_write_mode = g_settings_is_writable (client,
+                                                         KEY_MODE);
                 *is_writable = can_write_theme && can_write_mode;
         }
 
@@ -198,21 +176,18 @@ config_get_theme (gboolean *is_writable)
         } else if (mode == GS_MODE_RANDOM) {
                 name = g_strdup ("__random");
         } else {
-                GSList *list;
-                list = gconf_client_get_list (client,
-                                              KEY_THEMES,
-                                              GCONF_VALUE_STRING,
-                                              NULL);
-                if (list != NULL) {
-                        name = g_strdup (list->data);
+                gchar **strv;
+                strv = g_settings_get_strv (client,
+                                            KEY_THEMES);
+                if (strv != NULL) {
+                        name = g_strdup (strv[0]);
                 } else {
                         /* TODO: handle error */
                         /* default to blank */
                         name = g_strdup ("__blank-only");
                 }
 
-                g_slist_foreach (list, (GFunc)g_free, NULL);
-                g_slist_free (list);
+                g_strfreev (strv);
         }
 
         g_object_unref (client);
@@ -220,18 +195,19 @@ config_get_theme (gboolean *is_writable)
         return name;
 }
 
-static GSList *
+static gchar **
 get_all_theme_ids (GSThemeManager *theme_manager)
 {
-        GSList *ids = NULL;
+        gchar **ids = NULL;
         GSList *entries;
         GSList *l;
+        guint idx = 0;
 
         entries = gs_theme_manager_get_info_list (theme_manager);
+        ids = g_new0 (gchar *, g_slist_length (entries) + 1);
         for (l = entries; l; l = l->next) {
                 GSThemeInfo *info = l->data;
-
-                ids = g_slist_prepend (ids, g_strdup (gs_theme_info_get_id (info)));
+                ids[idx++] = g_strdup (gs_theme_info_get_id (info));
                 gs_theme_info_unref (info);
         }
         g_slist_free (entries);
@@ -242,11 +218,11 @@ get_all_theme_ids (GSThemeManager *theme_manager)
 static void
 config_set_theme (const char *theme_id)
 {
-        GConfClient *client;
-        GSList      *list = NULL;
+        GSettings *client;
+        gchar     **strv = NULL;
         int          mode;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
         if (theme_id && strcmp (theme_id, "__blank-only") == 0) {
                 mode = GS_MODE_BLANK_ONLY;
@@ -254,22 +230,19 @@ config_set_theme (const char *theme_id)
                 mode = GS_MODE_RANDOM;
 
                 /* set the themes key to contain all available screensavers */
-                list = get_all_theme_ids (theme_manager);
+                strv = get_all_theme_ids (theme_manager);
         } else {
                 mode = GS_MODE_SINGLE;
-                list = g_slist_append (list, g_strdup (theme_id));
+                strv = g_strsplit (theme_id, "%%%", 1);
         }
 
         config_set_mode (mode);
 
-        gconf_client_set_list (client,
-                               KEY_THEMES,
-                               GCONF_VALUE_STRING,
-                               list,
-                               NULL);
+        g_settings_set_strv (client,
+                             KEY_THEMES,
+                             (const gchar * const*) strv);
 
-        g_slist_foreach (list, (GFunc) g_free, NULL);
-        g_slist_free (list);
+        g_strfreev (strv);
 
         g_object_unref (client);
 }
@@ -278,17 +251,16 @@ static gboolean
 config_get_enabled (gboolean *is_writable)
 {
         int          enabled;
-        GConfClient *client;
+        GSettings *client;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
         if (is_writable) {
-                *is_writable = gconf_client_key_is_writable (client,
-                                                             KEY_LOCK,
-                                                             NULL);
+                *is_writable = g_settings_is_writable (client,
+                                                       KEY_LOCK);
         }
 
-        enabled = gconf_client_get_bool (client, KEY_IDLE_ACTIVATION_ENABLED, NULL);
+        enabled = g_settings_get_boolean (client, KEY_IDLE_ACTIVATION_ENABLED);
 
         g_object_unref (client);
 
@@ -298,11 +270,11 @@ config_get_enabled (gboolean *is_writable)
 static void
 config_set_enabled (gboolean enabled)
 {
-        GConfClient *client;
+        GSettings *client;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
-        gconf_client_set_bool (client, KEY_IDLE_ACTIVATION_ENABLED, enabled, NULL);
+        g_settings_set_boolean (client, KEY_IDLE_ACTIVATION_ENABLED, enabled);
 
         g_object_unref (client);
 }
@@ -310,18 +282,17 @@ config_set_enabled (gboolean enabled)
 static gboolean
 config_get_lock (gboolean *is_writable)
 {
-        GConfClient *client;
+        GSettings *client;
         gboolean     lock;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
         if (is_writable) {
-                *is_writable = gconf_client_key_is_writable (client,
-                                                             KEY_LOCK,
-                                                             NULL);
+                *is_writable = g_settings_is_writable (client,
+                                                       KEY_LOCK);
         }
 
-        lock = gconf_client_get_bool (client, KEY_LOCK, NULL);
+        lock = g_settings_get_boolean (client, KEY_LOCK);
 
         g_object_unref (client);
 
@@ -331,12 +302,12 @@ config_get_lock (gboolean *is_writable)
 static gboolean
 config_get_lock_disabled ()
 {
-        GConfClient *client;
+        GSettings *client;
         gboolean     lock;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (LOCKDOWN_SETTINGS_SCHEMA);
 
-        lock = gconf_client_get_bool (client, KEY_LOCK_DISABLE, NULL);
+        lock = g_settings_get_boolean (client, KEY_LOCK_DISABLE);
 
         g_object_unref (client);
         return lock;
@@ -345,11 +316,11 @@ config_get_lock_disabled ()
 static void
 config_set_lock (gboolean lock)
 {
-        GConfClient *client;
+        GSettings *client;
 
-        client = gconf_client_get_default ();
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
 
-        gconf_client_set_bool (client, KEY_LOCK, lock, NULL);
+        g_settings_set_boolean (client, KEY_LOCK, lock);
 
         g_object_unref (client);
 }
@@ -406,19 +377,15 @@ preview_set_theme (GtkWidget  *widget,
         if ((theme && strcmp (theme, "__blank-only") == 0)) {
 
         } else if (theme && strcmp (theme, "__random") == 0) {
-                GSList *themes;
+                gchar **themes;
 
                 themes = get_all_theme_ids (theme_manager);
                 if (themes != NULL) {
-                        GSList *l;
                         gint32  i;
 
-                        i = g_random_int_range (0, g_slist_length (themes));
-                        l = g_slist_nth (themes, i);
-
-                        job_set_theme (job, (const char *) l->data);
-                        g_slist_foreach (themes, (GFunc) g_free, NULL);
-                        g_slist_free (themes);
+                        i = g_random_int_range (0, g_strv_length (themes));
+                        job_set_theme (job, themes[i]);
+                        g_strfreev (themes);
 
                         gs_job_start (job);
                 }
@@ -431,38 +398,10 @@ preview_set_theme (GtkWidget  *widget,
 static void
 help_display (void)
 {
-        GError     *error = NULL;
-        char       *command;
-        const char *lang;
-        char       *uri = NULL;
-        GdkScreen  *gscreen;
-        int         i;
+        GError *error;
 
-        const char * const * langs = g_get_language_names ();
-
-        for (i = 0; langs[i] != NULL; i++) {
-                lang = langs[i];
-                if (strchr (lang, '.')) {
-                        continue;
-                }
-
-                uri = g_build_filename (DATADIR,
-                                        "/gnome/help/user-guide/",
-                                        lang,
-                                        "/user-guide.xml",
-                                        NULL);
-
-                if (g_file_test (uri, G_FILE_TEST_EXISTS)) {
-                    break;
-                }
-        }
-
-        command = g_strconcat ("gnome-open ghelp://",
-                               uri,
-                               "?prefs-screensaver",
-                               NULL);
-        gscreen = gdk_screen_get_default ();
-        gdk_spawn_command_line_on_screen (gscreen, command, &error);
+        error = NULL;
+        gtk_show_uri (NULL, "ghelp:prefs-screensaver", GDK_CURRENT_TIME, &error);
 
         if (error != NULL) {
                 GtkWidget *d;
@@ -475,9 +414,6 @@ help_display (void)
                 gtk_widget_destroy (d);
                 g_error_free (error);
         }
-
-        g_free (command);
-        g_free (uri);
 }
 
 static void
@@ -488,18 +424,25 @@ response_cb (GtkWidget *widget,
         if (response_id == GTK_RESPONSE_HELP) {
                 help_display ();
         } else if (response_id == GTK_RESPONSE_REJECT) {
+                GAppInfo *appinfo;
+                GAppLaunchContext *context;
                 GError  *error;
-                gboolean res;
 
                 error = NULL;
 
-                res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-                                                        GPM_COMMAND,
-                                                        &error);
-                if (! res) {
+                context = (GAppLaunchContext*)gdk_app_launch_context_new ();
+                appinfo = g_app_info_create_from_commandline (GPM_COMMAND, "Power preferences", 0, &error);
+                if (appinfo) {
+                        g_app_info_launch (appinfo, NULL, context, &error);
+                }
+
+                if (error) {
                         g_warning ("Unable to start power management preferences: %s", error->message);
                         g_error_free (error);
                 }
+
+                g_object_unref (context);
+                g_object_unref (appinfo);
         } else {
                 gtk_widget_destroy (widget);
                 gtk_main_quit ();
@@ -1039,13 +982,6 @@ enabled_checkbox_toggled (GtkToggleButton *button, gpointer user_data)
         config_set_enabled (gtk_toggle_button_get_active (button));
 }
 
-static void
-invalid_type_warning (const char *type)
-{
-        g_warning ("Error retrieving configuration key '%s': Invalid type",
-                   type);
-}
-
 static void
 ui_disable_lock (gboolean disable)
 {
@@ -1108,71 +1044,39 @@ ui_set_delay (int delay)
 }
 
 static void
-key_changed_cb (GConfClient *client,
-                guint        cnxn_id,
-                GConfEntry  *entry,
-                gpointer     data)
+key_changed_cb (GSettings   *settings,
+		const gchar *key,
+		gpointer     data)
 {
-        const char *key;
-        GConfValue *value;
-
-        key = gconf_entry_get_key (entry);
-
-        if (! g_str_has_prefix (key, KEY_DIR) && ! g_str_has_prefix (key, GNOME_LOCKDOWN_DIR)) {
-                return;
-        }
-
-        value = gconf_entry_get_value (entry);
-
         if (strcmp (key, KEY_IDLE_ACTIVATION_ENABLED) == 0) {
-                if (value->type == GCONF_VALUE_BOOL) {
-                        gboolean enabled;
+		gboolean enabled;
 
-                        enabled = gconf_value_get_bool (value);
+		enabled = g_settings_get_boolean (settings, key);
 
-                        ui_set_enabled (enabled);
-                } else {
-                        invalid_type_warning (key);
-                }
+		ui_set_enabled (enabled);
         } else if (strcmp (key, KEY_LOCK) == 0) {
-                if (value->type == GCONF_VALUE_BOOL) {
-                        gboolean enabled;
+		gboolean enabled;
 
-                        enabled = gconf_value_get_bool (value);
+		enabled = g_settings_get_boolean (settings, key);
 
-                        ui_set_lock (enabled);
-                } else {
-                        invalid_type_warning (key);
-                }
+		ui_set_lock (enabled);
         } else if (strcmp (key, KEY_LOCK_DISABLE) == 0) {
-                if (value->type == GCONF_VALUE_BOOL) {
-                        gboolean disabled;
+		gboolean disabled;
 
-                        disabled = gconf_value_get_bool (value);
+		disabled = g_settings_get_boolean (settings, key);
 
-                        ui_disable_lock (disabled);
-                } else {
-                        invalid_type_warning (key);
-                }
+		ui_disable_lock (disabled);
         } else if (strcmp (key, KEY_THEMES) == 0) {
-                if (value->type == GCONF_VALUE_LIST) {
-                        GtkWidget *treeview;
+		GtkWidget *treeview;
 
-                        treeview = GTK_WIDGET (gtk_builder_get_object (builder, "savers_treeview"));
-                        setup_treeview_selection (treeview);
-                } else {
-                        invalid_type_warning (key);
-                }
+		treeview = GTK_WIDGET (gtk_builder_get_object (builder, "savers_treeview"));
+		setup_treeview_selection (treeview);
         } else if (strcmp (key, KEY_ACTIVATE_DELAY) == 0) {
 
-                if (value->type == GCONF_VALUE_INT) {
-                        int delay;
+		int delay;
 
-                        delay = gconf_value_get_int (value);
-                        ui_set_delay (delay);
-                } else {
-                        invalid_type_warning (key);
-                }
+		delay = g_settings_get_int (settings, key);
+		ui_set_delay (delay);
 
         } else {
                 /*g_warning ("Config key not handled: %s", key);*/
@@ -1350,7 +1254,7 @@ get_best_visual (void)
                         VisualID      visual_id;
 
                         visual_id = (VisualID) v;
-                        visual = gdkx_visual_get (visual_id);
+                        visual = gdk_x11_screen_lookup_visual (gdk_screen_get_default (), visual_id);
 
                         g_debug ("Found best visual for GL: 0x%x",
                                  (unsigned int) visual_id);
@@ -1426,12 +1330,12 @@ init_capplet (void)
         GtkWidget *fullscreen_preview_area;
         GtkWidget *fullscreen_preview_close;
         char      *gtk_builder_file;
-        char      *string;
         gdouble    activate_delay;
         gboolean   enabled;
         gboolean   is_writable;
-        GConfClient *client;
+        GSettings *client;
         GError    *error=NULL;
+        gint       mode;
 
         gtk_builder_file = g_build_filename (GTKBUILDERDIR, GTK_BUILDER_FILE, NULL);
         builder = gtk_builder_new();
@@ -1529,38 +1433,19 @@ init_capplet (void)
         gtk_widget_show_all (dialog);
 
         /* Update list of themes if using random screensaver */
-        client = gconf_client_get_default ();
-        string = gconf_client_get_string (client, KEY_MODE, NULL);
-        if (string != NULL) {
-                int mode;
-                GSList *list;
+        client = g_settings_new (GS_SETTINGS_SCHEMA);
+        mode = g_settings_get_enum (client, KEY_MODE);
+	if (mode == GS_MODE_RANDOM) {
+		gchar **list;
+		list = get_all_theme_ids (theme_manager);
+		g_settings_set_strv (client, KEY_THEMES, (const gchar * const*) list);
+		g_strfreev (list);
+	}
 
-                gconf_string_to_enum (mode_enum_map, string, &mode);
-                g_free (string);
-
-                if (mode == GS_MODE_RANDOM) {
-                        list = get_all_theme_ids (theme_manager);
-                        gconf_client_set_list (client, KEY_THEMES, GCONF_VALUE_STRING, list, NULL);
-
-                        g_slist_foreach (list, (GFunc) g_free, NULL);
-                        g_slist_free (list);
-                }
-        }
-
-        gconf_client_add_dir (client, KEY_DIR,
-                              GCONF_CLIENT_PRELOAD_ONELEVEL,
-                              NULL);
-        gconf_client_notify_add (client,
-                                 KEY_DIR,
-                                 key_changed_cb,
-                                 NULL, NULL, NULL);
-        gconf_client_add_dir (client, GNOME_LOCKDOWN_DIR,
-                              GCONF_CLIENT_PRELOAD_ONELEVEL,
-                              NULL);
-        gconf_client_notify_add (client,
-                                 GNOME_LOCKDOWN_DIR,
-                                 key_changed_cb,
-                                 NULL, NULL, NULL);
+	g_signal_connect (client,
+			  "changed",
+			  G_CALLBACK (key_changed_cb),
+			  NULL);
 
         g_object_unref (client);
 
