@@ -36,7 +36,6 @@
 #include <gdk/gdkkeysyms.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
-#include <gconf/gconf-client.h>
 
 #ifdef WITH_KBD_LAYOUT_INDICATOR
 #include <libgnomekbd/gkbd-indicator.h>
@@ -153,8 +152,9 @@ gs_lock_plug_style_set (GtkWidget *widget,
 static void
 do_user_switch (GSLockPlug *plug)
 {
+        GAppInfo *app;
+        GAppLaunchContext *context;
         GError  *error;
-        gboolean res;
         char    *command;
 
         command = g_strdup_printf ("%s %s",
@@ -162,13 +162,16 @@ do_user_switch (GSLockPlug *plug)
                                    GDM_FLEXISERVER_ARGS);
 
         error = NULL;
-        res = gdk_spawn_command_line_on_screen (gdk_screen_get_default (),
-                                                command,
-                                                &error);
+        context = (GAppLaunchContext*)gdk_app_launch_context_new ();
+        app = g_app_info_create_from_commandline (command, "gdmflexiserver", 0, &error);
+        if (app)
+                g_app_info_launch (app, NULL, context, &error);
 
         g_free (command);
+        g_object_unref (context);
+        g_object_unref (app);
 
-        if (! res) {
+        if (!error) {
                 gs_debug ("Unable to start GDM greeter: %s", error->message);
                 g_error_free (error);
         }
@@ -870,28 +873,6 @@ forward_key_events (GSLockPlug *plug)
         }
 }
 
-static void
-gs_lock_plug_size_request (GtkWidget      *widget,
-                           GtkRequisition *requisition)
-{
-        int mod_width;
-        int mod_height;
-
-        if (GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->size_request) {
-                GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->size_request (widget, requisition);
-        }
-
-        mod_width = requisition->height * 1.3;
-        mod_height = requisition->width / 1.6;
-        if (requisition->width < mod_width) {
-                /* if the dialog is tall fill out the width */
-                requisition->width = mod_width;
-        } else if (requisition->height < mod_height) {
-                /* if the dialog is wide fill out the height */
-                requisition->height = mod_height;
-        }
-}
-
 static void
 gs_lock_plug_set_logout_enabled (GSLockPlug *plug,
                                  gboolean    logout_enabled)
@@ -1083,7 +1064,6 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
         widget_class->style_set    = gs_lock_plug_style_set;
         widget_class->show         = gs_lock_plug_show;
         widget_class->hide         = gs_lock_plug_hide;
-        widget_class->size_request = gs_lock_plug_size_request;
 
         klass->close = gs_lock_plug_close;
 
